const St = imports.gi.St;
const Lang = imports.lang;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Main = imports.ui.main;
const GLib = imports.gi.GLib;
const Util = imports.misc.util;
const Mainloop = imports.mainloop;

function NvidiaGpuTemperature() {
    this._init.apply(this, arguments);
}

NvidiaGpuTemperature.prototype = {
    //__proto__: PanelMenu.SystemStatusButton.prototype,
    __proto__: PanelMenu.Button.prototype,
	

    _init: function(){
        let menuAlignment = 0.25;
		PanelMenu.Button.prototype._init.call(this, 0);
        this.statusLabel = new St.Label({
            text: "UNSET", 
            style_class: "gputemperature-label"
        });

        // destroy all previously created children, and add our statusLabel
        this.actor.get_children().forEach(function(c) {
            c.destroy()
        });
        this.actor.add_actor(this.statusLabel);

		this.connect('button-press-event', this.showGpuInfo);
        this.getNvidiaGpuTemperature();
        //schedule recurring call to getNvidiaGpuTemperature every 15 seconds to update panel/menu
        event = GLib.timeout_add_seconds(0, 15, Lang.bind(this, function () {
            this.getNvidiaGpuTemperature();
            return true;
        }));
    },

	getNvidiaGpuTemperature: function()
	{
		this.menu.box.get_children().forEach(function(c) { c.destroy() });	
		items = new Array();

		gpu_command = "nvidia-smi -q "; // | grep '[[:space:]]*Gpu[[:space:]]*: [0-9]* C'";

		section = new PopupMenu.PopupMenuSection("GPU Temperature");
		
		gpuTemperatureTest = new RegExp("Gpu[ 	]*:[ 	][0-9]{1,3}[ 	]C","g");
		gpuTemperatureExtract = new RegExp("[0-9]{1,3}");
		driverVersionTest = new RegExp("^Driver Version");
		driverVersionExtract = new RegExp("[0-9]{1,10}\.[0-9]{0,10}");
		productNameTest = new RegExp("Product Name");
		productNameExtract = new RegExp(": [a-zA-Z0-9 ]{1,100}");
		fanSpeedTest = new RegExp("Fan Speed");
		fanSpeedExtract = new RegExp("[0-9]{0,2} %");
		
		
		try
		{
			gpu_sensors_output = GLib.spawn_command_line_sync(gpu_command);//get the output of the nvidia-smi command
			gpu_lines=gpu_sensors_output[1].toString().split("\n");

			
			totalTemperature = 0;
			totalGpus = 0;
			formattedTemperature = "??";
			dversion = "";
			let item;
			for(let i = 0; i < gpu_lines.length; i++) 
			{
				line = gpu_lines[i];
				if(gpuTemperatureTest.test(line))
				{
					totalTemperature += gpuTemperatureExtract.exec(line);
					totalGpus ++;
				}
				else if(driverVersionTest.test(line))
				{
					driverVersion = driverVersionExtract.exec(line);
					let item = new PopupMenu.PopupMenuItem("Driver Version: " + driverVersion.toString());
					section.addMenuItem(item);
				}
				else if(productNameTest.test(line))
				{
					productName = productNameExtract.exec(line);
					let item = new PopupMenu.PopupMenuItem("Video Card" + productName.toString());
					section.addMenuItem(item);
				}
				else if(fanSpeedTest.test(line))
				{
					fanSpeed = fanSpeedExtract.exec(line);
					let item = new PopupMenu.PopupMenuItem("Fan Speed: " + fanSpeed.toString());
					section.addMenuItem(item);
				}
			}
			if(totalGpus > 0)
			{
				temperature = totalTemperature / totalGpus;
				formattedTemperature = this.formatTemperature(temperature.toString());
			}
			
			// shows total # gpus if more than 1
			this.statusLabel.set_text(" GPU" + (totalGpus > 1?" (" + totalGpus + ")":"") + ": " + formattedTemperature + " ");
			
			this.menu.addMenuItem(section);
		}
		catch (e)
		{
			this.statusLabel.set_text(" GPU: ?? ");
			let item = new PopupMenu.PopupMenuItem("Your NVidia Driver doesn't seem to have");
			section.addMenuItem(item);
			item = new PopupMenu.PopupMenuItem("provided the status tool 'nvidia-smi' which");
			section.addMenuItem(item);
			item = new PopupMenu.PopupMenuItem("this extension uses to query the GPU.");
			section.addMenuItem(item);
			item = new PopupMenu.PopupMenuItem("Try reinstalling the NVidia Driver.");
			section.addMenuItem(item);
			this.menu.addMenuItem(section);
		}
        return 1;
	},
    
    formatTemperature: function(t) 
    {
        return (Math.round(t*10)/10).toString()+"\u1d3cC";
    },
    showGpuInfo: function()
    {
		
	},
}

let nvidiaGpuTemperature;

function init() {
}
function enable() {
		nvidiaGpuTemperature = new NvidiaGpuTemperature();
		Main.panel.addToStatusArea('gputemperature', nvidiaGpuTemperature);
}

function disable() {
		nvidiaGpuTemperature.destroy();

}
